@@ -21,12 +21,11 @@
 # SOFTWARE.
 
 
-import importlib_metadata
-
 from .rqdata_datafeed import RqdataDatafeed as Datafeed
+from .rqdata_gateway import RqdataGateway
 
 
-try:
-    __version__ = importlib_metadata.version("vnpy_rqdata")
-except importlib_metadata.PackageNotFoundError:
-    __version__ = "dev"
+__all__ = ["Datafeed", "RqdataGateway"]
+
+
+__version__ = "3.2.14.1"
