@@ -1,53 +1,71 @@
 from datetime import datetime, timedelta
-from typing import Dict, List, Optional
-from pytz import timezone
+from typing import cast
+from collections.abc import Callable
 
 from numpy import ndarray
-from pandas import DataFrame
+from pandas import DataFrame, Timestamp
 from rqdatac import init
 from rqdatac.services.get_price import get_price
+from rqdatac.services.future import get_dominant_price
 from rqdatac.services.basic import all_instruments
-from rqdatac.share.errors import AuthenticationFailed
+from rqdatac.services.calendar import get_next_trading_date
+from rqdatac.share.errors import RQDataError
 
 from vnpy.trader.setting import SETTINGS
 from vnpy.trader.constant import Exchange, Interval
 from vnpy.trader.object import BarData, TickData, HistoryRequest
-from vnpy.trader.utility import round_to
+from vnpy.trader.utility import round_to, ZoneInfo
 from vnpy.trader.datafeed import BaseDatafeed
 
 
-INTERVAL_VT2RQ: Dict[Interval, str] = {
+INTERVAL_VT2RQ: dict[Interval, str] = {
     Interval.MINUTE: "1m",
     Interval.HOUR: "60m",
     Interval.DAILY: "1d",
 }
 
-INTERVAL_ADJUSTMENT_MAP: Dict[Interval, timedelta] = {
+INTERVAL_ADJUSTMENT_MAP: dict[Interval, timedelta] = {
     Interval.MINUTE: timedelta(minutes=1),
     Interval.HOUR: timedelta(hours=1),
     Interval.DAILY: timedelta()         # no need to adjust for daily bar
 }
 
-CHINA_TZ = timezone("Asia/Shanghai")
+FUTURES_EXCHANGES: set[Exchange] = {
+    Exchange.CFFEX,
+    Exchange.SHFE,
+    Exchange.CZCE,
+    Exchange.DCE,
+    Exchange.INE,
+    Exchange.GFEX
+}
+
+CHINA_TZ = ZoneInfo("Asia/Shanghai")
 
 
-def to_rq_symbol(symbol: str, exchange: Exchange) -> str:
+def to_rq_symbol(symbol: str, exchange: Exchange, all_symbols: ndarray) -> str:
     """将交易所代码转换为米筐代码"""
     # 股票
-    if exchange in [Exchange.SSE, Exchange.SZSE]:
+    if exchange in {Exchange.SSE, Exchange.SZSE}:
         if exchange == Exchange.SSE:
             rq_symbol: str = f"{symbol}.XSHG"
         else:
-            rq_symbol: str = f"{symbol}.XSHE"
+            rq_symbol = f"{symbol}.XSHE"
     # 金交所现货
-    elif exchange in [Exchange.SGE]:
+    elif exchange == Exchange.SGE:
         for char in ["(", ")", "+"]:
-            symbol: str = symbol.replace(char, "")
+            symbol = symbol.replace(char, "")
         symbol = symbol.upper()
-        rq_symbol: str = f"{symbol}.SGEX"
+        rq_symbol = f"{symbol}.SGEX"
     # 期货和期权
-    elif exchange in [Exchange.SHFE, Exchange.CFFEX, Exchange.DCE, Exchange.CZCE, Exchange.INE]:
-        for count, word in enumerate(symbol):
+    elif exchange in {
+        Exchange.CFFEX,
+        Exchange.SHFE,
+        Exchange.DCE,
+        Exchange.CZCE,
+        Exchange.INE,
+        Exchange.GFEX
+    }:
+        for count, word in enumerate(symbol):  # noqa: B007
             if word.isdigit():
                 break
 
@@ -56,6 +74,7 @@ def to_rq_symbol(symbol: str, exchange: Exchange) -> str:
 
         # 期货
         if time_str.isdigit():
+            # 只有郑商所需要特殊处理
             if exchange is not Exchange.CZCE:
                 return symbol.upper()
 
@@ -63,31 +82,45 @@ def to_rq_symbol(symbol: str, exchange: Exchange) -> str:
             if time_str in ["88", "888", "99", "889"]:
                 return symbol
 
+            # 提取年月
             year: str = symbol[count]
             month: str = symbol[count + 1:]
 
-            if year == "9":
-                year = "1" + year
-            else:
-                year = "2" + year
+            guess_1: str = f"{product}1{year}{month}".upper()
+            guess_2: str = f"{product}2{year}{month}".upper()
 
-            rq_symbol: str = f"{product}{year}{month}".upper()
-        # 期权
+            # 优先尝试20年后的合约
+            if guess_2 in all_symbols:
+                rq_symbol = guess_2
+            else:
+                rq_symbol = guess_1
+        # 期权以及期货次主力连续合约
         else:
-            if exchange in [Exchange.CFFEX, Exchange.DCE, Exchange.SHFE]:
-                rq_symbol: str = symbol.replace("-", "").upper()
+            if time_str == "88A2":
+                return symbol.upper()
+
+            if exchange in {
+                Exchange.CFFEX,
+                Exchange.DCE,
+                Exchange.SHFE,
+                Exchange.INE,
+                Exchange.GFEX
+            }:
+                rq_symbol = symbol.replace("-", "").upper()
             elif exchange == Exchange.CZCE:
-                year: str = symbol[count]
+                year = symbol[count]
                 suffix: str = symbol[count + 1:]
 
-                if year == "9":
-                    year = "1" + year
-                else:
-                    year = "2" + year
+                guess_1 = f"{product}1{year}{suffix}".upper()
+                guess_2 = f"{product}2{year}{suffix}".upper()
 
-                rq_symbol: str = f"{product}{year}{suffix}".upper()
+                # 优先尝试20年后的合约
+                if guess_2 in all_symbols:
+                    rq_symbol = guess_2
+                else:
+                    rq_symbol = guess_1
     else:
-        rq_symbol: str = f"{symbol}.{exchange.value}"
+        rq_symbol = f"{symbol}.{exchange.value}"
 
     return rq_symbol
 
@@ -95,20 +128,24 @@ def to_rq_symbol(symbol: str, exchange: Exchange) -> str:
 class RqdataDatafeed(BaseDatafeed):
     """米筐RQData数据服务接口"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """"""
         self.username: str = SETTINGS["datafeed.username"]
         self.password: str = SETTINGS["datafeed.password"]
 
         self.inited: bool = False
-        self.symbols: ndarray = None
 
-    def init(self) -> bool:
+    def init(self, output: Callable = print) -> bool:
         """初始化"""
         if self.inited:
             return True
 
-        if not self.username or not self.password:
+        if not self.username:
+            output("RQData数据服务初始化失败：用户名为空！")
+            return False
+
+        if not self.password:
+            output("RQData数据服务初始化失败：密码为空！")
             return False
 
         try:
@@ -122,17 +159,32 @@ class RqdataDatafeed(BaseDatafeed):
             )
 
             df: DataFrame = all_instruments()
-            self.symbols = df["order_book_id"].values
-        except (RuntimeError, AuthenticationFailed):
+            self.symbols: ndarray = df["order_book_id"].to_numpy()
+        except RQDataError as ex:
+            output(f"RQData数据服务初始化失败：{ex}")
+            return False
+        except RuntimeError as ex:
+            output(f"发生运行时错误：{ex}")
+            return False
+        except Exception as ex:
+            output(f"发生未知异常：{ex}")
             return False
 
         self.inited = True
         return True
 
-    def query_bar_history(self, req: HistoryRequest) -> Optional[List[BarData]]:
+    def query_bar_history(self, req: HistoryRequest, output: Callable = print) -> list[BarData] | None:
+        """查询K线数据"""
+        # 期货品种且代码中没有数字（非具体合约），则查询主力连续
+        if req.exchange in FUTURES_EXCHANGES and req.symbol.isalpha():
+            return self._query_dominant_history(req, output)
+        else:
+            return self._query_bar_history(req, output)
+
+    def _query_bar_history(self, req: HistoryRequest, output: Callable = print) -> list[BarData] | None:
         """查询K线数据"""
         if not self.inited:
-            n: bool = self.init()
+            n: bool = self.init(output)
             if not n:
                 return []
 
@@ -142,41 +194,58 @@ class RqdataDatafeed(BaseDatafeed):
         start: datetime = req.start
         end: datetime = req.end
 
-        rq_symbol: str = to_rq_symbol(symbol, exchange)
+        # 股票期权不添加交易所后缀
+        if exchange in [Exchange.SSE, Exchange.SZSE] and symbol in self.symbols:
+            rq_symbol: str = symbol
+        else:
+            rq_symbol = to_rq_symbol(symbol, exchange, self.symbols)
 
-        rq_interval: str = INTERVAL_VT2RQ.get(interval)
+        # 检查查询的代码在范围内
+        if rq_symbol not in self.symbols:
+            output(f"RQData查询K线数据失败：不支持的合约代码{req.vt_symbol}")
+            return []
+
+        rq_interval: str | None = INTERVAL_VT2RQ.get(interval, None)
         if not rq_interval:
-            return None
+            output(f"RQData查询K线数据失败：不支持的时间周期{req.interval.value}")
+            return []
 
         # 为了将米筐时间戳（K线结束时点）转换为VeighNa时间戳（K线开始时点）
         adjustment: timedelta = INTERVAL_ADJUSTMENT_MAP[interval]
 
-        # 为了查询夜盘数据
-        end += timedelta(1)
-
         # 只对衍生品合约才查询持仓量数据
         fields: list = ["open", "high", "low", "close", "volume", "total_turnover"]
         if not symbol.isdigit():
             fields.append("open_interest")
 
+        # 对于股票查询前复权K线数据
+        if rq_symbol.endswith(".XSHG") or rq_symbol.endswith(".XSHE"):
+            adjust_type: str = "pre_volume"
+        else:
+            adjust_type = "none"
+
         df: DataFrame = get_price(
             rq_symbol,
             frequency=rq_interval,
             fields=fields,
             start_date=start,
-            end_date=end,
-            adjust_type="none"
+            end_date=get_next_trading_date(end),        # 为了查询夜盘数据
+            adjust_type=adjust_type
         )
 
-        data: List[BarData] = []
+        data: list[BarData] = []
 
         if df is not None:
             # 填充NaN为0
             df.fillna(0, inplace=True)
 
             for row in df.itertuples():
-                dt: datetime = row.Index[1].to_pydatetime() - adjustment
-                dt: datetime = CHINA_TZ.localize(dt)
+                row_index: tuple[str, Timestamp] = cast(tuple[str, Timestamp], row.Index)
+                dt: datetime = row_index[1].to_pydatetime() - adjustment
+                dt = dt.replace(tzinfo=CHINA_TZ)
+
+                if dt >= end:
+                    break
 
                 bar: BarData = BarData(
                     symbol=symbol,
@@ -197,10 +266,10 @@ class RqdataDatafeed(BaseDatafeed):
 
         return data
 
-    def query_tick_history(self, req: HistoryRequest) -> Optional[List[TickData]]:
+    def query_tick_history(self, req: HistoryRequest, output: Callable = print) -> list[TickData] | None:
         """查询Tick数据"""
         if not self.inited:
-            n: bool = self.init()
+            n: bool = self.init(output)
             if not n:
                 return []
 
@@ -209,12 +278,15 @@ class RqdataDatafeed(BaseDatafeed):
         start: datetime = req.start
         end: datetime = req.end
 
-        rq_symbol: str = to_rq_symbol(symbol, exchange)
-        if rq_symbol not in self.symbols:
-            return None
+        # 股票期权不添加交易所后缀
+        if exchange in [Exchange.SSE, Exchange.SZSE] and symbol in self.symbols:
+            rq_symbol: str = symbol
+        else:
+            rq_symbol = to_rq_symbol(symbol, exchange, self.symbols)
 
-        # 为了查询夜盘数据
-        end += timedelta(1)
+        if rq_symbol not in self.symbols:
+            output(f"RQData查询Tick数据失败：不支持的合约代码{req.vt_symbol}")
+            return []
 
         # 只对衍生品合约才查询持仓量数据
         fields: list = [
@@ -256,19 +328,23 @@ class RqdataDatafeed(BaseDatafeed):
             frequency="tick",
             fields=fields,
             start_date=start,
-            end_date=end,
+            end_date=get_next_trading_date(end),        # 为了查询夜盘数据
             adjust_type="none"
         )
 
-        data: List[TickData] = []
+        data: list[TickData] = []
 
         if df is not None:
             # 填充NaN为0
             df.fillna(0, inplace=True)
 
             for row in df.itertuples():
-                dt: datetime = row.Index[1].to_pydatetime()
-                dt: datetime = CHINA_TZ.localize(dt)
+                row_index: tuple[str, Timestamp] = cast(tuple[str, Timestamp], row.Index)
+                dt: datetime = row_index[1].to_pydatetime()
+                dt = dt.replace(tzinfo=CHINA_TZ)
+
+                if dt >= end:
+                    break
 
                 tick: TickData = TickData(
                     symbol=symbol,
@@ -310,3 +386,72 @@ class RqdataDatafeed(BaseDatafeed):
                 data.append(tick)
 
         return data
+
+    def _query_dominant_history(self, req: HistoryRequest, output: Callable = print) -> list[BarData] | None:
+        """查询期货主力K线数据"""
+        if not self.inited:
+            n: bool = self.init(output)
+            if not n:
+                return []
+
+        symbol: str = req.symbol
+        exchange: Exchange = req.exchange
+        interval: Interval = req.interval
+        start: datetime = req.start
+        end: datetime = req.end
+
+        rq_interval: str | None = INTERVAL_VT2RQ.get(interval, None)
+        if not rq_interval:
+            output(f"RQData查询K线数据失败：不支持的时间周期{req.interval.value}")
+            return []
+
+        # 为了将米筐时间戳（K线结束时点）转换为VeighNa时间戳（K线开始时点）
+        adjustment: timedelta = INTERVAL_ADJUSTMENT_MAP[interval]
+
+        # 只对衍生品合约才查询持仓量数据
+        fields: list = ["open", "high", "low", "close", "volume", "total_turnover"]
+        if not symbol.isdigit():
+            fields.append("open_interest")
+
+        df: DataFrame = get_dominant_price(
+            symbol.upper(),                         # 合约代码用大写
+            frequency=rq_interval,
+            fields=fields,
+            start_date=start,
+            end_date=get_next_trading_date(end),    # 为了查询夜盘数据
+            adjust_type="pre",                      # 前复权
+            adjust_method="prev_close_ratio"        # 切换前一日收盘价比例复权
+        )
+
+        data: list[BarData] = []
+
+        if df is not None:
+            # 填充NaN为0
+            df.fillna(0, inplace=True)
+
+            for row in df.itertuples():
+                row_index: tuple[str, Timestamp] = cast(tuple[str, Timestamp], row.Index)
+                dt: datetime = row_index[1].to_pydatetime() - adjustment
+                dt = dt.replace(tzinfo=CHINA_TZ)
+
+                if dt >= end:
+                    break
+
+                bar: BarData = BarData(
+                    symbol=symbol,
+                    exchange=exchange,
+                    interval=interval,
+                    datetime=dt,
+                    open_price=round_to(row.open, 0.000001),
+                    high_price=round_to(row.high, 0.000001),
+                    low_price=round_to(row.low, 0.000001),
+                    close_price=round_to(row.close, 0.000001),
+                    volume=row.volume,
+                    turnover=row.total_turnover,
+                    open_interest=getattr(row, "open_interest", 0),
+                    gateway_name="RQ"
+                )
+
+                data.append(bar)
+
+        return data
